@@ -1,4 +1,4 @@
-from NEMO.models import User
+from NEMO.models import User, Area
 
 
 def lock_selected_interlocks(model_admin, request, queryset):
@@ -14,7 +14,7 @@ def unlock_selected_interlocks(model_admin, request, queryset):
 def synchronize_with_tool_usage(model_admin, request, queryset):
 	for interlock in queryset:
 		# Ignore interlocks with no tool assigned, and ignore interlocks connected to doors
-		if not interlock.tool or interlock.door:
+		if not hasattr(interlock, 'tool') or hasattr(interlock, 'door'):
 			continue
 		if interlock.tool.in_use():
 			interlock.unlock()
@@ -42,3 +42,6 @@ def duplicate_tool_configuration(model_admin, request, queryset):
 			tool.backup_owners.set(old_backup_users)
 			for user in old_qualified_users:
 				user.qualifications.add(tool)
+
+def rebuild_area_tree(model_admin, request, queryset):
+	Area.objects.rebuild()
\ No newline at end of file
