@@ -1,3 +1,4 @@
+import sys
 from json import loads
 from django import forms
 from django.contrib import admin
@@ -5,19 +6,20 @@ from django.contrib.admin import register
 from django.contrib.admin.widgets import FilteredSelectMultiple
 from django.contrib.auth.models import Permission
 from django.db.models.fields.files import FieldFile
-from django.utils.html import format_html
+from django.utils.safestring import mark_safe
 from mptt.admin import DraggableMPTTAdmin, TreeRelatedFieldListFilter
 
 from NEMO.actions import lock_selected_interlocks, synchronize_with_tool_usage, unlock_selected_interlocks, \
-	duplicate_tool_configuration
+	duplicate_tool_configuration, rebuild_area_tree
 from NEMO.models import Account, ActivityHistory, Alert, Area, AreaAccessRecord, Comment, Configuration, \
-	ConfigurationHistory, Consumable, ConsumableCategory, ConsumableWithdraw, ContactInformation, \
+	ConfigurationHistory, Consumable, ConsumableCategory, ConsumableWithdraw, ContactInformation, BadgeReader, \
 	ContactInformationCategory, Customization, Door, Interlock, InterlockCard, LandingPageChoice, MembershipHistory, \
 	News, Notification, PhysicalAccessLevel, PhysicalAccessLog, Project, Reservation, Resource, ResourceCategory, \
 	SafetyIssue, ScheduledOutage, ScheduledOutageCategory, StaffCharge, Task, TaskCategory, TaskHistory, TaskStatus, \
 	Tool, TrainingSession, UsageEvent, User, UserType, UserPreferences, TaskImages, InterlockCardCategory, \
-	record_remote_many_to_many_changes_and_save, record_local_many_to_many_changes, record_active_state, AlertCategory
-from NEMO.widgets.dynamic_form import DynamicForm
+	record_remote_many_to_many_changes_and_save, record_local_many_to_many_changes, record_active_state, AlertCategory, \
+	ToolUsageCounter, PhysicalAccessException
+from NEMO.widgets.dynamic_form import DynamicForm, PostUsageNumberFieldQuestion
 
 
 class ToolAdminForm(forms.ModelForm):
@@ -113,7 +115,12 @@ class ToolAdminForm(forms.ModelForm):
 					loads(post_usage_questions)
 				except ValueError as error:
 					self.add_error("_post_usage_questions", "This field needs to be a valid JSON string")
-					
+				try:
+					DynamicForm(post_usage_questions, self.instance.id).validate()
+				except Exception:
+					error_info = sys.exc_info()
+					self.add_error("_post_usage_questions", error_info[0].__name__ + ": " + str(error_info[1]))
+
 			policy_off_between_times = cleaned_data.get("_policy_off_between_times")
 			policy_off_start_time = cleaned_data.get("_policy_off_start_time")
 			policy_off_end_time = cleaned_data.get("_policy_off_end_time")
@@ -144,8 +151,9 @@ class ToolAdmin(admin.ModelAdmin):
 	)
 
 	def _post_usage_preview(self, obj):
-		form_validity_div = '<div id="form_validity"></div>' if obj.post_usage_questions else ''
-		return format_html('<div class="post_usage_preview">{}{}</div><div class="help post_usage_preview_help">Save form to preview post usage questions</div>'.format(DynamicForm(obj.post_usage_questions).render(), form_validity_div))
+		if obj.id:
+			form_validity_div = '<div id="form_validity"></div>' if obj.post_usage_questions else ''
+			return mark_safe('<div class="post_usage_preview">{}{}</div><div class="help post_usage_preview_help">Save form to preview post usage questions</div>'.format(DynamicForm(obj.post_usage_questions, obj.id).render(), form_validity_div))
 
 	def formfield_for_foreignkey(self, db_field, request, **kwargs):
 		""" We only want non children tool to be eligible as parents """
@@ -178,13 +186,14 @@ class AreaAdmin(DraggableMPTTAdmin):
 	fieldsets = (
 		(None, {'fields': ('name', 'parent_area', 'category', 'reservation_email', 'abuse_email'),}),
 		('Area access', {'fields': ('requires_reservation', 'logout_grace_period', 'welcome_message'),}),
-		('Occupancy', {'fields': ('maximum_capacity', 'count_staff_in_occupancy', 'reservation_warning'),}),
+		('Occupancy', {'fields': ('maximum_capacity', 'count_staff_in_occupancy', 'count_service_personnel_in_occupancy', 'reservation_warning'),}),
 		('Reservation', {'fields': ('reservation_horizon', 'missed_reservation_threshold'),}),
 		('Policy', {'fields': ('policy_off_between_times', 'policy_off_start_time', 'policy_off_end_time', 'policy_off_weekend', 'minimum_usage_block_time', 'maximum_usage_block_time', 'maximum_reservations_per_day', 'minimum_time_between_reservations', 'maximum_future_reservation_time',),}),
 	)
 	list_display_links = ('indented_title',)
 	list_filter = ('requires_reservation', ('parent_area', TreeRelatedFieldListFilter))
 	search_fields = ('name',)
+	actions = [rebuild_area_tree]
 
 	mptt_level_indent = 20
 
@@ -307,6 +316,7 @@ class ProjectAdmin(admin.ModelAdmin):
 @register(Reservation)
 class ReservationAdmin(admin.ModelAdmin):
 	list_display = ('id', 'user', 'creator', 'tool', 'project', 'start', 'end', 'duration', 'cancelled', 'missed')
+	readonly_fields = ('descendant',)
 	list_filter = ('cancelled', 'missed', 'tool')
 	date_hierarchy = 'start'
 
@@ -322,6 +332,7 @@ class UsageEventAdmin(admin.ModelAdmin):
 class ConsumableAdmin(admin.ModelAdmin):
 	list_display = ('name', 'quantity', 'category', 'visible', 'reminder_threshold', 'reminder_email', 'id')
 	list_filter = ('visible', 'category')
+	readonly_fields = ('reminder_threshold_reached',)
 
 
 @register(ConsumableCategory)
@@ -457,18 +468,36 @@ class UserPreferencesAdmin(admin.ModelAdmin):
 	list_display = ('user',)
 
 
+class UserAdminForm(forms.ModelForm):
+	class Meta:
+		model = User
+		fields = '__all__'
+
+	def clean(self):
+		cleaned_data = super().clean()
+		staff_status = cleaned_data.get("is_staff")
+		service_personnel_status = cleaned_data.get("is_service_personnel")
+
+		if staff_status and service_personnel_status:
+			raise forms.ValidationError({
+				'is_staff': 'A user cannot be both staff and service personnel. Please choose one or the other.',
+				'is_service_personnel': 'A user cannot be both staff and service personnel. Please choose one or the other.',
+			})
+
+
 @register(User)
 class UserAdmin(admin.ModelAdmin):
+	form = UserAdminForm
 	filter_horizontal = ('groups', 'user_permissions', 'qualifications', 'projects', 'physical_access_levels')
 	fieldsets = (
 		('Personal information', {'fields': ('first_name', 'last_name', 'username', 'email', 'badge_number', 'type', 'domain')}),
-		('Permissions', {'fields': ('is_active', 'is_staff', 'is_technician', 'is_superuser', 'training_required', 'groups', 'user_permissions', 'physical_access_levels')}),
+		('Permissions', {'fields': ('is_active', 'is_staff', 'is_technician', 'is_service_personnel', 'is_superuser', 'training_required', 'groups', 'user_permissions', 'physical_access_levels')}),
 		('Important dates', {'fields': ('date_joined', 'last_login', 'access_expiration')}),
 		("Facility information", {'fields': ('qualifications', 'projects')}),
 	)
 	search_fields = ('first_name', 'last_name', 'username', 'email')
-	list_display = ('first_name', 'last_name', 'username', 'email', 'is_active', 'domain', 'is_staff', 'is_technician', 'is_superuser', 'date_joined', 'last_login')
-	list_filter = ('is_active', 'domain', 'is_staff', 'is_technician', 'is_superuser', 'date_joined', 'last_login')
+	list_display = ('first_name', 'last_name', 'username', 'email', 'is_active', 'domain', 'is_staff', 'is_technician', 'is_service_personnel', 'is_superuser', 'date_joined', 'last_login')
+	list_filter = ('is_active', 'domain', 'is_staff', 'is_technician', 'is_service_personnel', 'is_superuser', 'date_joined', 'last_login')
 
 	def formfield_for_manytomany(self, db_field, request, **kwargs):
 		if db_field.name == "qualifications":
@@ -488,7 +517,7 @@ class UserAdmin(admin.ModelAdmin):
 class PhysicalAccessLogAdmin(admin.ModelAdmin):
 	list_display = ('user', 'door', 'time', 'result')
 	list_filter = ('door', 'result')
-	search_fields = ('user',)
+	search_fields = ('user__first_name', 'user__last_name', 'user__username', 'door__name')
 	date_hierarchy = 'time'
 
 
@@ -541,10 +570,20 @@ class PhysicalAccessLevelForm(forms.ModelForm):
 		)
 	)
 
+	physical_access_exceptions = forms.ModelMultipleChoiceField(
+		queryset=PhysicalAccessException.objects.all(),
+		required=False,
+		widget=FilteredSelectMultiple(
+			verbose_name='Physical Access Exceptions',
+			is_stacked=False
+		)
+	)
+
 	def __init__(self, *args, **kwargs):
-		super(PhysicalAccessLevelForm, self).__init__(*args, **kwargs)
+		super().__init__(*args, **kwargs)
 		if self.instance.pk:
 			self.fields['authorized_users'].initial = self.instance.user_set.all()
+			self.fields['physical_access_exceptions'].initial = self.instance.physicalaccessexception_set.all()
 
 	def clean(self):
 		schedule = self.cleaned_data.get('schedule')
@@ -571,7 +610,31 @@ class PhysicalAccessLevelAdmin(admin.ModelAdmin):
 		"""
 		Explicitly record any membership changes.
 		"""
-		record_remote_many_to_many_changes_and_save(request, obj, form, change, 'authorized_users', super(PhysicalAccessLevelAdmin, self).save_model)
+		record_remote_many_to_many_changes_and_save(request, obj, form, change, 'authorized_users', super().save_model)
+		if 'physical_access_exceptions' in form.changed_data:
+			obj.physicalaccessexception_set.set(form.cleaned_data['physical_access_exceptions'])
+
+
+class PhysicalAccessExceptionAdminForm(forms.ModelForm):
+	class Meta:
+		model = PhysicalAccessException
+		fields = '__all__'
+
+	physical_access_levels = forms.ModelMultipleChoiceField(
+		queryset=PhysicalAccessLevel.objects.all(),
+		required=False,
+		widget=FilteredSelectMultiple(
+			verbose_name='Physical Access Levels',
+			is_stacked=False
+		)
+	)
+
+
+@register(PhysicalAccessException)
+class PhysicalAccessExceptionAdmin(admin.ModelAdmin):
+	form = PhysicalAccessExceptionAdminForm
+	list_display = ('name', 'start_time', 'end_time')
+	list_filter = (('physical_access_levels__area'),)
 
 
 @register(ContactInformationCategory)
@@ -581,7 +644,7 @@ class ContactInformationCategoryAdmin(admin.ModelAdmin):
 
 @register(ContactInformation)
 class ContactInformationAdmin(admin.ModelAdmin):
-	list_display = ('name', 'category')
+	list_display = ('name', 'category', 'user')
 
 
 @register(LandingPageChoice)
@@ -616,5 +679,42 @@ class NotificationAdmin(admin.ModelAdmin):
 	list_display = ('id', 'user', 'expiration', 'content_type', 'object_id')
 
 
+@register(BadgeReader)
+class BadgeReaderAdmin(admin.ModelAdmin):
+	list_display = ('id', 'name', 'send_key', 'record_key')
+
+
+class CounterAdminForm(forms.ModelForm):
+	class Meta:
+		model = ToolUsageCounter
+		fields = '__all__'
+
+	def clean(self):
+		cleaned_data = super().clean()
+		tool = cleaned_data.get('tool')
+		tool_usage_question_name = cleaned_data.get('tool_usage_question')
+		if tool and tool_usage_question_name:
+			error = None
+			if tool.post_usage_questions:
+				post_usage_form = DynamicForm(tool.post_usage_questions, tool.id)
+				tool_question = post_usage_form.filter_questions(lambda x: isinstance(x, PostUsageNumberFieldQuestion) and x.name == tool_usage_question_name)
+				if not tool_question:
+					candidates = [question.name for question in post_usage_form.filter_questions(lambda x: isinstance(x, PostUsageNumberFieldQuestion))]
+					error = "The tool has no post usage question of type Number with this name."
+					if candidates:
+						error += f" Valid question names are: {', '.join(candidates)}"
+			else:
+				error = "The tool does not have any post usage questions."
+			if error:
+				self.add_error('tool_usage_question', error)
+		return cleaned_data
+
+
+@register(ToolUsageCounter)
+class CounterAdmin(admin.ModelAdmin):
+	list_display = ('name', 'tool', 'tool_usage_question', 'value', 'last_reset', 'last_reset_by', 'is_active')
+	form = CounterAdminForm
+
+
 admin.site.register(ResourceCategory)
 admin.site.register(Permission)
