@@ -8,7 +8,7 @@ from django.utils import timezone
 from django.utils.dateparse import parse_date, parse_time
 from django.views.decorators.http import require_GET, require_POST
 
-from NEMO.models import Project, Reservation, Tool, UsageEvent, User
+from NEMO.models import Project, Reservation, Tool, UsageEvent, User, BadgeReader
 from NEMO.utilities import quiet_int, localize
 from NEMO.views.calendar import determine_insufficient_notice, extract_configuration, cancel_the_reservation, shorten_reservation
 from NEMO.views.policy import check_policy_to_disable_tool, check_policy_to_enable_tool, \
@@ -78,9 +78,10 @@ def disable_tool(request):
 	current_usage_event.end = timezone.now() + downtime
 
 	# Collect post-usage questions
-	dynamic_form = DynamicForm(tool.post_usage_questions)
+	dynamic_form = DynamicForm(tool.post_usage_questions, tool.id)
 	current_usage_event.run_data = dynamic_form.extract(request)
 	dynamic_form.charge_for_consumables(current_usage_event.user, current_usage_event.operator, current_usage_event.project, current_usage_event.run_data)
+	dynamic_form.update_counters(current_usage_event.run_data)
 
 	current_usage_event.save()
 	dictionary = {
@@ -184,12 +185,10 @@ def tool_reservation(request, tool_id, user_id, back):
 def choices(request):
 	try:
 		customer = User.objects.get(badge_number=request.GET['badge_number'])
-		usage_events = UsageEvent.objects.filter(operator=customer.id, end=None).prefetch_related('tool', 'project')
+		usage_events = UsageEvent.objects.filter(operator=customer.id, end=None).order_by('tool__name').prefetch_related('tool', 'project')
 		tools_in_use = [u.tool.tool_or_parent_id() for u in usage_events]
 		fifteen_minutes_from_now = timezone.now() + timedelta(minutes=15)
-		reservations = Reservation.objects.filter(end__gt=timezone.now(), user=customer, missed=False, cancelled=False, shortened=False).exclude(tool_id__in=tools_in_use, start__lte=fifteen_minutes_from_now).order_by('start')
-		if customer.in_area():
-			reservations = reservations.exclude(area=customer.area_access_record().area, start__lte=fifteen_minutes_from_now)
+		tool_reservations = Reservation.objects.filter(tool__isnull=False, end__gt=timezone.now(), user=customer, missed=False, cancelled=False, shortened=False).exclude(tool_id__in=tools_in_use, start__lte=fifteen_minutes_from_now).order_by('start')
 	except:
 		dictionary = {'message': "Your badge wasn't recognized. If you got a new one recently then we'll need to update your account. Please contact staff to resolve the problem."}
 		return render(request, 'kiosk/acknowledgement.html', dictionary)
@@ -199,8 +198,8 @@ def choices(request):
 	dictionary = {
 		'now': timezone.now(),
 		'customer': customer,
-		'usage_events': UsageEvent.objects.filter(operator=customer.id, end=None).order_by('tool__name').prefetch_related('tool', 'project'),
-		'upcoming_reservations': reservations,
+		'usage_events': list(usage_events),
+		'upcoming_reservations': tool_reservations,
 		'tool_summary': create_tool_summary(),
 		'categories': categories,
 		'unqualified_categories': unqualified_categories,
@@ -238,7 +237,7 @@ def tool_information(request, tool_id, user_id, back):
 		'customer': customer,
 		'tool': tool,
 		'rendered_configuration_html': tool.configuration_widget(customer),
-		'post_usage_questions': DynamicForm(tool.post_usage_questions).render(),
+		'post_usage_questions': DynamicForm(questions=tool.post_usage_questions, tool_id=tool.id, is_mobile=True).render(),
 		'back': back,
 	}
 	try:
@@ -258,8 +257,10 @@ def tool_information(request, tool_id, user_id, back):
 @require_GET
 def kiosk(request, location=None):
 	if location and Tool.objects.filter(_location=location, visible=True).exists():
+		reader_id = request.GET.get("reader_id")
 		dictionary = {
 			'location': location,
+			'badge_reader': BadgeReader.objects.get(id=reader_id) if reader_id else BadgeReader.default(),
 		}
 		return render(request, 'kiosk/kiosk.html', dictionary)
 	else:
