@@ -3,6 +3,8 @@ import os
 import sys
 from datetime import timedelta
 from enum import Enum
+from html import escape
+from logging import getLogger
 from typing import Union, List
 
 from django.conf import settings
@@ -14,6 +16,8 @@ from django.db import models
 from django.db.models import Q
 from django.db.models.signals import pre_delete
 from django.dispatch import receiver
+from django.template import loader
+from django.urls import reverse
 from django.utils import timezone
 from mptt.fields import TreeForeignKey
 from mptt.models import MPTTModel
@@ -23,6 +27,8 @@ from NEMO.utilities import send_mail, get_task_image_filename, get_tool_image_fi
 from NEMO.views.constants import ADDITIONAL_INFORMATION_MAXIMUM_LENGTH
 from NEMO.widgets.configuration_editor import ConfigurationEditor
 
+models_logger = getLogger(__name__)
+
 
 class ReservationItemType(Enum):
 	TOOL = 'tool'
@@ -128,6 +134,7 @@ class User(models.Model):
 	# Permissions
 	is_active = models.BooleanField('active', default=True, help_text='Designates whether this user can log in. Unselect this instead of deleting accounts.')
 	is_staff = models.BooleanField('staff status', default=False, help_text='Designates whether the user can log into this admin site.')
+	is_service_personnel = models.BooleanField('service personnel', default=False, help_text='Designates this user as service personnel. Service personnel can operate qualified tools without a reservation even when they are shutdown or during an outage and can access authorized areas without a reservation.')
 	is_technician = models.BooleanField('technician status', default=False, help_text='Specifies how to bill staff time for this user. When checked, customers are billed at technician rates.')
 	is_superuser = models.BooleanField('superuser status', default=False, help_text='Designates that this user has all permissions without explicitly assigning them.')
 	training_required = models.BooleanField(default=True, help_text='When selected, the user is blocked from all reservation and tool usage capabilities.')
@@ -208,16 +215,22 @@ class User(models.Model):
 	def has_usable_password(self):
 		return False
 
+	def set_unusable_password(self):
+		pass
+
 	def email_user(self, subject, message, from_email, attachments=None):
 		""" Sends an email to this user. """
 		send_mail(subject=subject, message=message, from_email=from_email, recipient_list=[self.email], attachments=attachments)
 
 	def get_full_name(self):
-		return self.first_name + ' ' + self.last_name + ' (' + self.username + ')'
+		return self.get_name() + ' (' + self.username + ')'
 
 	def get_short_name(self):
 		return self.first_name
 
+	def get_name(self):
+		return self.first_name + ' ' + self.last_name
+
 	def accessible_access_levels(self):
 		if not self.is_staff:
 			return self.physical_access_levels.all()
@@ -281,6 +294,15 @@ class User(models.Model):
 			self.save()
 		return self.preferences
 
+	def get_contact_info_html(self):
+		if hasattr(self, 'contactinformation'):
+			content = escape(loader.render_to_string('contact/contact_person.html', {'person':self.contactinformation, 'email_form': True}))
+			return f'<a href="javascript:;" data-title="{content}" data-placement="bottom" class="contact-info-tooltip"><span class="glyphicon glyphicon-send small-icon"></span>{self.contactinformation.name}</a>'
+		else:
+			email_url = reverse('get_email_form_for_user', kwargs={'user_id':self.id})
+			return f'<a href="{email_url}" title="Email {self.first_name}"><span class="glyphicon glyphicon-send small-icon"></span>{self.get_name()}</a>'
+
+
 	@classmethod
 	def get_email_field_name(cls):
 		return 'email'
@@ -379,7 +401,7 @@ class Tool(models.Model):
 		self._operational = value
 
 	@property
-	def primary_owner(self):
+	def primary_owner(self) -> User:
 		return self.parent_tool.primary_owner if self.is_child_tool() else self._primary_owner
 
 	@primary_owner.setter
@@ -388,7 +410,7 @@ class Tool(models.Model):
 		self._primary_owner = value
 
 	@property
-	def backup_owners(self):
+	def backup_owners(self) -> List[User]:
 		return self.parent_tool.backup_owners if self.is_child_tool() else self._backup_owners
 
 	@backup_owners.setter
@@ -741,6 +763,9 @@ class Tool(models.Model):
 	def requires_area_reservation(self):
 		return self.requires_area_access and self.requires_area_access.requires_reservation
 
+	def active_counters(self):
+		return self.toolusagecounter_set.filter(is_active=True)
+
 
 class Configuration(models.Model):
 	tool = models.ForeignKey(Tool, help_text="The tool that this configuration option applies to.", on_delete=models.CASCADE)
@@ -850,6 +875,7 @@ class Area(MPTTModel):
 	# Capacity
 	maximum_capacity = models.PositiveIntegerField(help_text='The maximum number of people allowed in this area at any given time. Set to 0 for unlimited.', default=0)
 	count_staff_in_occupancy = models.BooleanField(default=True, help_text='Indicates that staff users will count towards maximum capacity.')
+	count_service_personnel_in_occupancy = models.BooleanField(default=True, help_text='Indicates that service personnel will count towards maximum capacity.')
 	reservation_warning = models.PositiveIntegerField(blank=True, null=True, help_text='The number of simultaneous users (with at least one reservation in this area) allowed before a warning is displayed when creating a reservation.')
 
 	# Policy rules
@@ -908,14 +934,19 @@ class Area(MPTTModel):
 
 	def occupancy_count(self):
 		""" Returns the occupancy used to determine if the area is at capacity """
-		if self.count_staff_in_occupancy:
-			return self.occupancy()
-		else:
-			return self.occupancy() - self.occupancy_staff()
+		result = self.occupancy()
+		if not self.count_staff_in_occupancy:
+			result = result - self.occupancy_staff()
+		if not self.count_service_personnel_in_occupancy:
+			result = result - self.occupancy_service_personnel()
+		return result
 
 	def occupancy_staff(self):
 		return AreaAccessRecord.objects.filter(area__in=self.get_descendants(include_self=True), end=None, staff_charge=None, customer__is_staff=True).count()
 
+	def occupancy_service_personnel(self):
+		return AreaAccessRecord.objects.filter(area__in=self.get_descendants(include_self=True), end=None, staff_charge=None, customer__is_service_personnel=True).count()
+
 	def occupancy(self):
 		return AreaAccessRecord.objects.filter(area__in=self.get_descendants(include_self=True), end=None, staff_charge=None).count()
 
@@ -1120,6 +1151,7 @@ class Consumable(models.Model):
 	visible = models.BooleanField(default=True)
 	reminder_threshold = models.IntegerField(help_text="More of this item should be ordered when the quantity falls below this threshold.")
 	reminder_email = models.EmailField(help_text="An email will be sent to this address when the quantity of this item falls below the reminder threshold.")
+	reminder_threshold_reached = models.BooleanField(default=False)
 
 	class Meta:
 		ordering = ['name']
@@ -1127,6 +1159,22 @@ class Consumable(models.Model):
 	def __str__(self):
 		return self.name
 
+# This method is used to check when the quantity of a consumable falls below the threshold and when it has been replenished
+@receiver(models.signals.pre_save, sender=Consumable)
+def check_consumable_quantity_threshold(sender, instance: Consumable, **kwargs):
+	try:
+		if not instance.reminder_threshold_reached and instance.quantity < instance.reminder_threshold:
+			# quantity is below threshold. set flag and send email
+			instance.reminder_threshold_reached = True
+			from NEMO.views.consumables import send_reorder_supply_reminder_email
+			send_reorder_supply_reminder_email(instance)
+		if instance.reminder_threshold_reached and instance.quantity >= instance.reminder_threshold:
+			# it has been replenished. reset flag
+			instance.reminder_threshold_reached = False
+	except Exception as e:
+		models_logger.exception(e)
+		pass
+
 
 class ConsumableCategory(models.Model):
 	name = models.CharField(max_length=100)
@@ -1559,7 +1607,7 @@ class PhysicalAccessLevel(models.Model):
 		if self.schedule == self.Schedule.ALWAYS or self.schedule == self.Schedule.WEEKENDS:
 			return self.get_schedule_display()
 		else:
-			return self.get_schedule_display() + f" from {self.weekdays_start_time.strftime('%-I:%M %p')} to {self.weekdays_end_time.strftime('%-I:%M %p')}"
+			return self.get_schedule_display() + f" from {self.weekdays_start_time.strftime('%I:%M %p')} to {self.weekdays_end_time.strftime('%I:%M %p')}"
 	get_schedule_display_with_times.short_description = 'Schedule'
 
 	def accessible_at(self, time):
@@ -1570,6 +1618,10 @@ class PhysicalAccessLevel(models.Model):
 			accessible_time = timezone.localtime(time)
 		else:
 			accessible_time = timezone.localtime(timezone.now())
+		# First deal with exceptions
+		if self.ongoing_exception(accessible_time):
+			return False
+		# Then look at the actual allowed schedule
 		saturday = 6
 		sunday = 7
 		if self.schedule == self.Schedule.ALWAYS:
@@ -1591,6 +1643,13 @@ class PhysicalAccessLevel(models.Model):
 				return True
 		return False
 
+	def ongoing_exception(self, time: datetime = None):
+		if time is not None:
+			accessible_time = timezone.localtime(time)
+		else:
+			accessible_time = timezone.localtime(timezone.now())
+		return self.physicalaccessexception_set.filter(start_time__lte=accessible_time, end_time__gt=accessible_time).first()
+
 	def __str__(self):
 		return str(self.name)
 
@@ -1598,6 +1657,18 @@ class PhysicalAccessLevel(models.Model):
 		ordering = ['name']
 
 
+class PhysicalAccessException(models.Model):
+	name = models.CharField(max_length=100, help_text="The name of this exception that will be displayed as the policy problem")
+	start_time = models.DateTimeField(help_text="The start of the exception, after which users will be denied access.")
+	end_time = models.DateTimeField(help_text="The end of the exception, after which users will be allowed access again")
+	physical_access_levels = models.ManyToManyField('PhysicalAccessLevel', blank=True)
+
+	class Meta:
+		ordering = ['-start_time']
+
+	def __str__(self):
+		return str(self.name)
+
 class PhysicalAccessType(object):
 	DENY = False
 	ALLOW = True
@@ -1693,6 +1764,7 @@ class ContactInformation(models.Model):
 	office_location = models.CharField(max_length=200, blank=True)
 	mobile_phone = models.CharField(max_length=40, blank=True)
 	mobile_phone_is_sms_capable = models.BooleanField(default=True, verbose_name='Mobile phone is SMS capable', help_text="Is the mobile phone capable of receiving text messages? If so, a link will be displayed for users to click to send a text message to the recipient when viewing the 'Contact information' page.")
+	user = models.OneToOneField(to=User, blank=True, null=True, help_text="Select a user to associate with this contact. When set, this contact information will be shown instead of the user information on pages like tool details.", on_delete=models.CASCADE)
 
 	class Meta:
 		verbose_name_plural = 'Contact information'
@@ -1818,6 +1890,37 @@ class News(models.Model):
 		verbose_name_plural = 'News'
 
 
+class BadgeReader(models.Model):
+	name = models.CharField(max_length=200)
+	send_key = models.CharField(max_length=20, help_text="The name of the key which submits the badge number ('F2', 'Shift', 'Meta', 'Enter', 'a' etc.)")
+	record_key = models.CharField(null=True, blank=True, max_length=20, help_text="The name of the key which starts badge number recording. If left blank, badge number recording starts when any input is received.")
+
+	def __str__(self):
+		return str(self.name)
+
+	@staticmethod
+	def default():
+		# The default badge reader is a badge reader using F2 for recording and sending
+		default_badge_reader = BadgeReader()
+		default_badge_reader.record_key = "F2"
+		default_badge_reader.send_key = "F2"
+		return default_badge_reader
+
+
+class ToolUsageCounter(models.Model):
+	name = models.CharField(max_length=200, help_text="The name of this counter")
+	description = models.TextField(null=True, blank=True, help_text="The counter description to be displayed next to it on the tool control page")
+	value = models.PositiveIntegerField(default=0, help_text="The current value of this counter")
+	tool = models.ForeignKey(Tool, help_text="The tool this counter is for.", on_delete=models.CASCADE)
+	tool_usage_question = models.CharField(max_length=200, help_text="The name of the tool's post usage question which should be used to increment this counter")
+	last_reset_value = models.PositiveIntegerField(null=True, blank=True, help_text="The last value before the counter was reset")
+	last_reset = models.DateTimeField(null=True, blank=True, help_text="The date and time this counter was last reset")
+	last_reset_by = models.ForeignKey(User, null=True, blank=True, help_text="The user who last reset this counter", on_delete=models.SET_NULL)
+	is_active = models.BooleanField(default=True, help_text="The state of the counter")
+
+	def __str__(self):
+		return str(self.name)
+
 def record_remote_many_to_many_changes_and_save(request, obj, form, change, many_to_many_field, save_function_pointer):
 	"""
 	Record the changes in a many-to-many field that the model does not own. Then, save the many-to-many field.
