@@ -1,5 +1,5 @@
 import io
-from collections import Iterable, defaultdict
+from collections import Iterable
 from copy import deepcopy
 from datetime import timedelta, datetime
 from http import HTTPStatus
@@ -22,7 +22,7 @@ from NEMO.models import Tool, Reservation, Configuration, UsageEvent, AreaAccess
 from NEMO.utilities import bootstrap_primary_color, extract_times, extract_dates, format_datetime, parse_parameter_string, send_mail, create_email_attachment, localize
 from NEMO.views.constants import ADDITIONAL_INFORMATION_MAXIMUM_LENGTH
 from NEMO.views.customization import get_customization, get_media_file_contents
-from NEMO.views.policy import check_policy_to_save_reservation, check_policy_to_cancel_reservation, check_policy_to_create_outage
+from NEMO.views.policy import check_policy_to_save_reservation, check_policy_to_cancel_reservation, check_policy_to_create_outage, maximum_users_in_overlapping_reservations, check_tool_reservation_requiring_area
 
 calendar_logger = getLogger(__name__)
 
@@ -71,6 +71,7 @@ def calendar(request, item_type=None, item_id=None):
 	calendar_week_column_format = get_customization('calendar_week_column_format')
 	calendar_month_column_format = get_customization('calendar_month_column_format')
 	calendar_start_of_the_day = get_customization('calendar_start_of_the_day')
+	calendar_now_indicator = get_customization('calendar_now_indicator')
 
 	dictionary = {
 		'rendered_item_tree_html': rendered_item_tree_html,
@@ -84,6 +85,7 @@ def calendar(request, item_type=None, item_id=None):
 		'calendar_week_column_format' : calendar_week_column_format,
 		'calendar_month_column_format' : calendar_month_column_format,
 		'calendar_start_of_the_day' : calendar_start_of_the_day,
+		'calendar_now_indicator' : calendar_now_indicator,
 		'self_login': False,
 		'self_logout': False,
 	}
@@ -170,37 +172,36 @@ def reservation_event_feed(request, start, end):
 
 
 def usage_event_feed(request, start, end):
-	usage_events = UsageEvent.objects
+	usage_events = UsageEvent.objects.none()
+	area_access_events = AreaAccessRecord.objects.none()
+	missed_reservations = Reservation.objects.none()
+
+	item_id = request.GET.get('item_id')
+	item_type = ReservationItemType(request.GET.get('item_type')) if request.GET.get('item_type') else None
+
+	personal_schedule = request.GET.get('personal_schedule')
+	if personal_schedule:
+		# Filter events that only have to do with the current user.
+		# Display missed reservations, tool and area usage when 'personal schedule' is selected
+		usage_events = UsageEvent.objects.filter(user=request.user)
+		area_access_events = AreaAccessRecord.objects.filter(customer=request.user)
+		missed_reservations = Reservation.objects.filter(missed=True, user=request.user)
+	elif item_type:
+		reservation_filter = {item_type.value: item_id}
+		missed_reservations = Reservation.objects.filter(missed=True).filter(**reservation_filter)
+		# Filter events that only have to do with the relevant tool or area.
+		if item_id and item_type == ReservationItemType.TOOL:
+			usage_events = UsageEvent.objects.filter(tool__id__in=Tool.objects.get(pk=item_id).get_family_tool_ids())
+		if item_id and item_type == ReservationItemType.AREA:
+			area_access_events = AreaAccessRecord.objects.filter(area__id=item_id)
+
 	# Exclude events for which the following is true:
 	# The event starts and ends before the time-window, and...
 	# The event starts and ends after the time-window.
 	usage_events = usage_events.exclude(start__lt=start, end__lt=start)
 	usage_events = usage_events.exclude(start__gt=end, end__gt=end)
-
-	# Filter events that only have to do with the relevant tool.
-	item_id = request.GET.get('item_id')
-	item_type = ReservationItemType(request.GET.get('item_type')) if request.GET.get('item_type') else None
-	if item_id and item_type == ReservationItemType.TOOL:
-		usage_events = usage_events.filter(tool__id__in=Tool.objects.get(pk=item_id).get_family_tool_ids())
-
-	area_access_events = AreaAccessRecord.objects.none()
-	# Filter events that only have to do with the current user.
-	personal_schedule = request.GET.get('personal_schedule')
-	if personal_schedule:
-		usage_events = usage_events.filter(user=request.user)
-		# Display area access along side tool usage when 'personal schedule' is selected.
-		area_access_events = AreaAccessRecord.objects.filter(customer__id=request.user.id)
-	if item_id and item_type == ReservationItemType.AREA:
-		area_access_events = AreaAccessRecord.objects.filter(area__id=item_id)
 	area_access_events = area_access_events.exclude(start__lt=start, end__lt=start)
 	area_access_events = area_access_events.exclude(start__gt=end, end__gt=end)
-
-	missed_reservations = Reservation.objects.none()
-	if personal_schedule:
-		missed_reservations = Reservation.objects.filter(missed=True, user=request.user)
-	elif item_type:
-		reservation_filter = {item_type.value: item_id}
-		missed_reservations = Reservation.objects.filter(missed=True).filter(**reservation_filter)
 	missed_reservations = missed_reservations.exclude(start__lt=start, end__lt=start)
 	missed_reservations = missed_reservations.exclude(start__gt=end, end__gt=end)
 
@@ -348,10 +349,10 @@ def reservation_success(request, reservation: Reservation):
 			overlapping_reservations_in_same_area = overlapping_reservations_in_same_area.filter(tool__in=Tool.objects.filter(_requires_area_access=area))
 		elif reservation.reservation_item_type == ReservationItemType.AREA:
 			overlapping_reservations_in_same_area = overlapping_reservations_in_same_area.filter(area=area)
-		max_area_overlap, max_area_time = maximum_overlap_users(overlapping_reservations_in_same_area)
+		max_area_overlap, max_area_time = maximum_users_in_overlapping_reservations(overlapping_reservations_in_same_area)
 		if location:
 			overlapping_reservations_in_same_location = overlapping_reservations_in_same_area.filter(tool__in=Tool.objects.filter(_location=location))
-			max_location_overlap, max_location_time = maximum_overlap_users(overlapping_reservations_in_same_location)
+			max_location_overlap, max_location_time = maximum_users_in_overlapping_reservations(overlapping_reservations_in_same_location)
 	if max_area_overlap and max_area_overlap >= area.warning_capacity():
 		dictionary = {
 			'area': area,
@@ -523,16 +524,8 @@ def modify_reservation(request, start_delta, end_delta):
 		explicit_policy_override = request.POST['explicit_policy_override'] == 'true'
 	except:
 		pass
-	response = check_policy_to_cancel_reservation(reservation_to_cancel, request.user)
-	# Do not move the reservation if the user was not authorized to cancel it.
-	if response.status_code != HTTPStatus.OK:
-		return response
 	# Record the current time so that the timestamp of the cancelled reservation and the new reservation match exactly.
 	now = timezone.now()
-	# Cancel the user's original reservation.
-	reservation_to_cancel.cancelled = True
-	reservation_to_cancel.cancellation_time = now
-	reservation_to_cancel.cancelled_by = request.user
 	# Create a new reservation for the user.
 	new_reservation = Reservation()
 	new_reservation.title = reservation_to_cancel.title
@@ -555,6 +548,17 @@ def modify_reservation(request, start_delta, end_delta):
 	new_reservation.project = reservation_to_cancel.project
 	new_reservation.user = reservation_to_cancel.user
 	new_reservation.creation_time = now
+
+	response = check_policy_to_cancel_reservation(request.user, reservation_to_cancel, new_reservation)
+	# Do not move the reservation if the user was not authorized to cancel it.
+	if response.status_code != HTTPStatus.OK:
+		return response
+
+	# Cancel the user's original reservation.
+	reservation_to_cancel.cancelled = True
+	reservation_to_cancel.cancellation_time = now
+	reservation_to_cancel.cancelled_by = request.user
+
 	policy_problems, overridable = check_policy_to_save_reservation(cancelled_reservation=reservation_to_cancel, new_reservation=new_reservation, user_creating_reservation=request.user, explicit_policy_override=explicit_policy_override)
 	if policy_problems:
 		reservation_action = "resize" if start_delta is None else "move"
@@ -650,12 +654,15 @@ def change_reservation_project(request, reservation_id):
 @permission_required('NEMO.trigger_timed_services', raise_exception=True)
 @require_GET
 def email_reservation_reminders(request):
+	send_email_reservation_reminders()
+
+def send_email_reservation_reminders():
 	# Exit early if the reservation reminder email template has not been customized for the organization yet.
 	reservation_reminder_message = get_media_file_contents('reservation_reminder_email.html')
 	reservation_warning_message = get_media_file_contents('reservation_warning_email.html')
 	if not reservation_reminder_message or not reservation_warning_message:
-		calendar_logger.error("Reservation reminder email couldn't be send because reservation_reminder_email.html is not defined")
-		return HttpResponseNotFound('The reservation reminder email template has not been customized for your organization yet. Please visit the customization page to upload a template, then reservation reminder email notifications can be sent.')
+		calendar_logger.error("Reservation reminder email couldn't be send because either reservation_reminder_email.html or reservation_warning_email.html is not defined")
+		return HttpResponseNotFound('The reservation reminder and/or warning email templates have not been customized for your organization yet. Please visit the customization page to upload both templates, then reservation reminder email notifications can be sent.')
 
 	# Find all reservations that are two hours from now, plus or minus 5 minutes to allow for time skew.
 	preparation_time = 120
@@ -687,6 +694,11 @@ def email_reservation_reminders(request):
 @require_GET
 def email_usage_reminders(request):
 	projects_to_exclude = request.GET.getlist("projects_to_exclude[]")
+	send_email_usage_reminders(projects_to_exclude)
+
+def send_email_usage_reminders(projects_to_exclude=None):
+	if projects_to_exclude is None:
+		projects_to_exclude = []
 	busy_users = AreaAccessRecord.objects.filter(end=None, staff_charge=None).exclude(project__id__in=projects_to_exclude)
 	busy_tools = UsageEvent.objects.filter(end=None).exclude(project__id__in=projects_to_exclude)
 
@@ -769,15 +781,15 @@ def area_access_details(request, event_id):
 @require_GET
 @permission_required('NEMO.trigger_timed_services', raise_exception=True)
 def cancel_unused_reservations(request):
+	do_cancel_unused_reservations()
+
+def do_cancel_unused_reservations():
 	"""
 	Missed reservation for tools is when there is no tool activity during the reservation time + missed reservation threshold.
 	Any tool usage will count, since we don't want to charge for missed reservation when users swap reservation or somebody else gets to use the tool.
 
-	Missed reservation for areas is then there is no area access login during the reservation time + missed reservation threshold
+	Missed reservation for areas is when there is no area access login during the reservation time + missed reservation threshold
 	"""
-	# Exit early if the missed reservation email template has not been customized for the organization yet.
-	if not get_media_file_contents('missed_reservation_email.html'):
-		return HttpResponseNotFound('The missed reservation email template has not been customized for your organization yet. Please visit the customization page to upload a template, then missed email notifications can be sent.')
 
 	# Missed Tool Reservations
 	tools = Tool.objects.filter(visible=True, _operational=True, _missed_reservation_threshold__isnull=False)
@@ -835,10 +847,14 @@ def cancel_unused_reservations(request):
 @require_GET
 @permission_required('NEMO.trigger_timed_services', raise_exception=True)
 def email_out_of_time_reservation_notification(request):
+	send_email_out_of_time_reservation_notification()
+
+def send_email_out_of_time_reservation_notification():
 	"""
 	Out of time reservation notification for areas is when a user is still logged in a area but his reservation expired.
 	"""
-	# Exit early if the missed reservation email template has not been customized for the organization yet.
+	# Exit early if the out of time reservation email template has not been customized for the organization yet.
+	# This feature only sends emails, so there if the template is not defined there nothing to do.
 	if not get_media_file_contents('out_of_time_reservation_email.html'):
 		return HttpResponseNotFound('The out of time reservation email template has not been customized for your organization yet. Please visit the customization page to upload a template, then out of time email notifications can be sent.')
 
@@ -847,19 +863,21 @@ def email_out_of_time_reservation_notification(request):
 	# Find all logged users
 	access_records:List[AreaAccessRecord] = AreaAccessRecord.objects.filter(end=None, staff_charge=None).prefetch_related('customer', 'area').only('customer', 'area')
 	for access_record in access_records:
-		# staff are exempt from out of time notification
+		# staff and service personnel are exempt from out of time notification
 		customer = access_record.customer
 		area = access_record.area
-		if customer.is_staff:
+		if customer.is_staff or customer.is_service_personnel:
 			continue
 
 		if area.requires_reservation:
 			# Calculate the timestamp of how late a user can be logged in after a reservation ended.
 			threshold = timezone.now() if not area.logout_grace_period else timezone.now() - timedelta(minutes=area.logout_grace_period)
 			threshold = datetime.replace(threshold, second=0, microsecond=0)  # Round down to the nearest minute.
-			reservations = Reservation.objects.filter(cancelled=False, missed=False, shortened=False, area=area, user=customer, start__lte=timezone.now(), end=threshold)
-			if reservations.exists():
-				out_of_time_user_area.append(reservations[0])
+			ending_reservations = Reservation.objects.filter(cancelled=False, missed=False, shortened=False, area=area, user=customer, start__lte=timezone.now(), end=threshold)
+			# find out if a reservation is starting right at the same time (in case of back to back reservations, in which case customer is good)
+			starting_reservations = Reservation.objects.filter(cancelled=False, missed=False, shortened=False, area=area, user=customer, start=threshold)
+			if ending_reservations.exists() and not starting_reservations.exists():
+				out_of_time_user_area.append(ending_reservations[0])
 
 	for reservation in out_of_time_user_area:
 		send_out_of_time_reservation_notification(reservation)
@@ -895,11 +913,19 @@ def shorten_reservation(user: User, item: Union[Area, Tool], new_end: datetime =
 
 
 def cancel_the_reservation(reservation: Reservation, user_cancelling_reservation: User, reason: Optional[str]):
-	response = check_policy_to_cancel_reservation(reservation, user_cancelling_reservation)
+	# Check policy to cancel reservation contains rules common to cancelling and modifying
+	response = check_policy_to_cancel_reservation(user_cancelling_reservation, reservation)
+
+	# The following rules apply only for proper cancellation, not for modification
 	# Staff must provide a reason when cancelling a reservation they do not own.
 	if reservation.user != user_cancelling_reservation and not reason:
 		response = HttpResponseBadRequest("You must provide a reason when cancelling someone else's reservation.")
 
+	policy_problems = []
+	check_tool_reservation_requiring_area(policy_problems, user_cancelling_reservation, reservation, None)
+	if policy_problems:
+		return HttpResponseBadRequest(policy_problems[0])
+
 	if response.status_code == HTTPStatus.OK:
 		# All policy checks passed, so cancel the reservation.
 		reservation.cancelled = True
@@ -907,16 +933,16 @@ def cancel_the_reservation(reservation: Reservation, user_cancelling_reservation
 		reservation.cancelled_by = user_cancelling_reservation
 
 		if reason:
-			''' don't notify in this case since we are sending a specific email for the cancellation '''
+			''' don't notify (just save) in this case since we are sending a specific email for the cancellation '''
 			reservation.save()
-			dictionary = {
-				'staff_member': user_cancelling_reservation,
-				'reservation': reservation,
-				'reason': reason,
-				'template_color': bootstrap_primary_color('info')
-			}
 			email_contents = get_media_file_contents('cancellation_email.html')
 			if email_contents:
+				dictionary = {
+					'staff_member': user_cancelling_reservation,
+					'reservation': reservation,
+					'reason': reason,
+					'template_color': bootstrap_primary_color('info')
+				}
 				cancellation_email = Template(email_contents).render(Context(dictionary))
 				recipients = [reservation.user.email]
 				if reservation.area:
@@ -935,11 +961,11 @@ def cancel_the_reservation(reservation: Reservation, user_cancelling_reservation
 
 
 def send_missed_reservation_notification(reservation):
-	subject = "Missed reservation for the " + str(reservation.reservation_item)
 	message = get_media_file_contents('missed_reservation_email.html')
 	user_office_email = get_customization('user_office_email_address')
 	abuse_email = get_customization('abuse_email_address')
 	if message and user_office_email:
+		subject = "Missed reservation for the " + str(reservation.reservation_item)
 		message = Template(message).render(Context({'reservation': reservation}))
 		send_mail(subject, message, user_office_email, [reservation.user.email, abuse_email, user_office_email])
 	else:
@@ -947,10 +973,10 @@ def send_missed_reservation_notification(reservation):
 
 
 def send_out_of_time_reservation_notification(reservation:Reservation):
-	subject = "Out of time in the " + str(reservation.area.name)
 	message = get_media_file_contents('out_of_time_reservation_email.html')
 	user_office_email = get_customization('user_office_email_address')
 	if message and user_office_email:
+		subject = "Out of time in the " + str(reservation.area.name)
 		message = Template(message).render(Context({'reservation': reservation}))
 		recipients = [reservation.user.email]
 		recipients.extend(reservation.area.abuse_email_list())
@@ -1014,54 +1040,3 @@ def create_ics_for_reservation(reservation: Reservation, cancelled=False):
 	filename = 'cancelled_reservation.ics' if cancelled else 'reservation.ics'
 
 	return create_email_attachment(ics, filename)
-
-
-def maximum_overlap_users(reservations: List[Reservation]) -> (int, datetime):
-	"""
-	Returns the maximum number of overlapping reservations and the earlier time the maximum is reached
-	This will only count reservations made by different users. i.e. if a user has 3 reservations at the same
-	time for different tools, it will only count as one.
-	"""
-	# First we need to merge reservations by user, since one user could have more than one at the same time. (and we should only count it as one)
-	intervals_by_user = defaultdict(list)
-	for r in reservations:
-		intervals_by_user[r.user.id].append((r.start, r.end))
-
-	merged_intervals = []
-	for user, intervals in intervals_by_user.items():
-		merged_intervals.extend(recursive_merge(sorted(intervals).copy()))
-
-	# Now let's count the maximum overlapping reservations
-	times = []
-	for interval in merged_intervals:
-		start_time, end_time = interval[0], interval[1]
-		times.append((start_time, 'start'))
-		times.append((end_time, 'end'))
-	times = sorted(times)
-
-	count = 0
-	max_count = 0
-	max_time = None
-	for time in times:
-		if time[1] == 'start':
-			count += 1  # increment on arrival/start
-		else:
-			count -= 1  # decrement on departure/end
-		# maintain maximum
-		prev_count = max_count
-		max_count = max(count, max_count)
-		# maintain earlier time max is reached
-		if max_count > prev_count:
-			max_time = time[0]
-	return max_count, max_time
-
-
-def recursive_merge(intervals: List[tuple], start_index=0) -> List[tuple]:
-	for i in range(start_index, len(intervals) - 1):
-		if intervals[i][1] > intervals[i + 1][0]:
-			new_start = intervals[i][0]
-			new_end = intervals[i + 1][1]
-			intervals[i] = (new_start, new_end)
-			del intervals[i + 1]
-			return recursive_merge(intervals.copy(), start_index=i)
-	return intervals
