@@ -49,6 +49,7 @@ customizable_key_values = {
 	'calendar_week_column_format': 'ddd M/DD',
 	'calendar_month_column_format': 'ddd',
 	'calendar_start_of_the_day': '07:00:00',
+	'calendar_now_indicator': '',
 	'calendar_display_not_qualified_areas': '',
 }
 
@@ -73,6 +74,7 @@ customizable_content = [
 	('usage_reminder_email', '.html'),
 	('reservation_cancelled_user_email', '.html'),
 	('reservation_created_user_email', '.html'),
+	('reorder_supplies_reminder_email', '.html'),
 	('rates', '.json'),
 	('jumbotron_watermark', '.png'),
 ]
@@ -146,6 +148,7 @@ def customize(request, element):
 		set_customization('calendar_view', request.POST.get('calendar_view', ''))
 		set_customization('calendar_first_day_of_week', request.POST.get('calendar_first_day_of_week', ''))
 		set_customization('calendar_start_of_the_day', request.POST.get('calendar_start_of_the_day', ''))
+		set_customization('calendar_now_indicator', request.POST.get('calendar_now_indicator', ''))
 		set_customization('calendar_day_column_format', request.POST.get('calendar_day_column_format', ''))
 		set_customization('calendar_week_column_format', request.POST.get('calendar_week_column_format', ''))
 		set_customization('calendar_month_column_format', request.POST.get('calendar_month_column_format', ''))
