@@ -1,15 +1,18 @@
-from datetime import timedelta, date
+from collections import defaultdict
+from datetime import timedelta, date, datetime
 from typing import Optional, List
 
 from django.db.models import Q
 from django.http import HttpResponse, HttpResponseBadRequest
 from django.template import Template, Context
 from django.utils import timezone
+from django.utils.formats import localize
 
 from NEMO.exceptions import InactiveUserError, NoActiveProjectsForUserError, PhysicalAccessExpiredUserError, \
 	NoPhysicalAccessUserError, NoAccessiblePhysicalAccessUserError, UnavailableResourcesUserError, \
 	MaximumCapacityReachedError, ReservationRequiredUserError, ScheduledOutageInProgressError
-from NEMO.models import Reservation, AreaAccessRecord, ScheduledOutage, User, Area, PhysicalAccessLevel, ReservationItemType, Tool, Project
+from NEMO.models import Reservation, AreaAccessRecord, ScheduledOutage, User, Area, PhysicalAccessLevel, \
+	ReservationItemType, Tool, Project, PhysicalAccessException
 from NEMO.utilities import format_datetime, send_mail
 from NEMO.views.customization import get_customization, get_media_file_contents
 
@@ -26,8 +29,8 @@ def check_policy_to_enable_tool(tool: Tool, operator: User, user: User, project:
 		return HttpResponseBadRequest("This tool is currently hidden from users.")
 
 	# The tool must be operational.
-	# If the tool is non-operational then it may only be accessed by staff members.
-	if not tool.operational and not operator.is_staff:
+	# If the tool is non-operational then it may only be accessed by staff members or service personnel.
+	if not tool.operational and not operator.is_staff and not operator.is_service_personnel:
 		return HttpResponseBadRequest("This tool is currently non-operational.")
 
 	# The tool must not be in use.
@@ -44,36 +47,37 @@ def check_policy_to_enable_tool(tool: Tool, operator: User, user: User, project:
 	if (user and operator.pk != user.pk) and not operator.is_staff:
 		return HttpResponseBadRequest("You must be a staff member to use a tool on another user's behalf.")
 
-	# All required resources must be available to operate a tool except for staff.
-	if tool.required_resource_set.filter(available=False).exists() and not operator.is_staff:
+	# All required resources must be available to operate a tool except for staff or service personnel.
+	if tool.required_resource_set.filter(available=False).exists() and not operator.is_staff and not operator.is_service_personnel:
 		return HttpResponseBadRequest("A resource that is required to operate this tool is unavailable.")
 
 	# The tool operator may not activate tools in a particular area unless they are logged in to the area.
 	# Staff are exempt from this rule.
 	if tool.requires_area_access and AreaAccessRecord.objects.filter(area=tool.requires_area_access, customer=operator, staff_charge=None, end=None).count() == 0 and not operator.is_staff:
-		dictionary = {
-			'operator': operator,
-			'tool': tool,
-			'type': 'access'
-		}
 		abuse_email_address = get_customization('abuse_email_address')
 		message = get_media_file_contents('unauthorized_tool_access_email.html')
 		if abuse_email_address and message:
+			dictionary = {
+				'operator': operator,
+				'tool': tool,
+				'type': 'access'
+			}
 			rendered_message = Template(message).render(Context(dictionary))
 			send_mail("Area access requirement", rendered_message, abuse_email_address, [abuse_email_address])
 		return HttpResponseBadRequest("You must be logged in to the {} to operate this tool.".format(tool.requires_area_access.name))
 
 	# The tool operator may not activate tools in a particular area unless they are still within that area reservation window
-	if not operator.is_staff and tool.requires_area_reservation():
+	# Staff and service personnel are exempt from this rule.
+	if not operator.is_staff and not operator.is_service_personnel and tool.requires_area_reservation():
 		if not tool.requires_area_access.get_current_reservation_for_user(operator):
-			dictionary = {
-				'operator': operator,
-				'tool': tool,
-				'type': 'reservation',
-			}
 			abuse_email_address = get_customization('abuse_email_address')
 			message = get_media_file_contents('unauthorized_tool_access_email.html')
 			if abuse_email_address and message:
+				dictionary = {
+					'operator': operator,
+					'tool': tool,
+					'type': 'reservation',
+				}
 				rendered_message = Template(message).render(Context(dictionary))
 				send_mail("Area reservation requirement", rendered_message, abuse_email_address, [abuse_email_address])
 			return HttpResponseBadRequest("You must have a current reservation for the {} to operate this tool.".format(tool.requires_area_access.name))
@@ -82,7 +86,7 @@ def check_policy_to_enable_tool(tool: Tool, operator: User, user: User, project:
 	if staff_charge and operator.charging_staff_time():
 		return HttpResponseBadRequest('You are already charging staff time. You must end the current staff charge before you being another.')
 
-	# Staff may not bill staff time to the themselves.
+	# Staff may not bill staff time to themselves.
 	if staff_charge and operator == user:
 		return HttpResponseBadRequest('You cannot charge staff time to yourself.')
 
@@ -94,12 +98,12 @@ def check_policy_to_enable_tool(tool: Tool, operator: User, user: User, project:
 	if operator.training_required:
 		return HttpResponseBadRequest(f"You are blocked from using all tools in the {facility_name}. Please complete the {facility_name} rules tutorial in order to use tools.")
 
-	# Users may only use a tool when delayed logoff is not in effect. Staff are exempt from this rule.
-	if tool.delayed_logoff_in_progress() and not operator.is_staff:
+	# Users may only use a tool when delayed logoff is not in effect. Staff and service personnel are exempt from this rule.
+	if tool.delayed_logoff_in_progress() and not operator.is_staff and not operator.is_service_personnel:
 		return HttpResponseBadRequest("Delayed tool logoff is in effect. You must wait for the delayed logoff to expire before you can use the tool.")
 
-	# Users may not enable a tool during a scheduled outage. Staff are exempt from this rule.
-	if tool.scheduled_outage_in_progress() and not operator.is_staff:
+	# Users may not enable a tool during a scheduled outage. Staff and service personnel are exempt from this rule.
+	if tool.scheduled_outage_in_progress() and not operator.is_staff and not operator.is_service_personnel:
 		return HttpResponseBadRequest("A scheduled outage is in effect. You must wait for the outage to end before you can use the tool.")
 
 	return HttpResponse()
@@ -199,9 +203,11 @@ def check_policy_to_save_reservation(cancelled_reservation: Optional[Reservation
 		policy_problems.append("You may not change reservations that you do not own.")
 
 	# The user may not create or move a reservation to have a start time that is earlier than the current time.
+	# Unless it's an extension of an area reservation, in which case the start time is the same.
 	# Staff may break this rule.
 	# An explicit policy override allows this rule to be broken.
-	if new_reservation.start < timezone.now():
+	extension_of_area_reservation = new_reservation.area and cancelled_reservation and cancelled_reservation.start == new_reservation.start
+	if not extension_of_area_reservation and new_reservation.start < timezone.now():
 		policy_problems.append("Reservation start time (" + format_datetime(new_reservation.start) + ") is earlier than the current time (" + format_datetime(timezone.now()) + ").")
 
 	# The user may not move or resize a reservation to have an end time that is earlier than the current time.
@@ -225,12 +231,22 @@ def check_policy_to_save_reservation(cancelled_reservation: Optional[Reservation
 	if item_type == ReservationItemType.AREA:
 		user_access_levels = user.accessible_access_levels_for_area(new_reservation.area)
 		if not any([access_level.accessible_at(new_reservation.start) for access_level in user_access_levels]) or not any([access_level.accessible_at(new_reservation.end) for access_level in user_access_levels]):
-			details = f" (times allowed in this area are: {','.join([access.get_schedule_display_with_times() for access in user_access_levels])})" if user_access_levels else ''
+			# it could be inaccessible because of an ongoing exception at the start or end time
+			first_access_exception: PhysicalAccessException = next(iter([access_level.ongoing_exception(new_reservation.start) for access_level in user_access_levels]), None)
+			if not first_access_exception:
+				first_access_exception = next(iter([access_level.ongoing_exception(new_reservation.end) for access_level in user_access_levels]), None)
+			if first_access_exception:
+				details = f" due to the following exception: {first_access_exception.name} (from {localize(first_access_exception.start_time.astimezone(timezone.get_current_timezone()))} to {localize(first_access_exception.end_time.astimezone(timezone.get_current_timezone()))}"
+			# or simply due to scheduling
+			else:
+				details = f" (times allowed in this area are: {', '.join([access.get_schedule_display_with_times() for access in user_access_levels])})" if user_access_levels else ''
 			if user == user_creating_reservation:
 				policy_problems.append(f"You are not authorized to access this area at this time{details}. Creating, moving, and resizing reservations is forbidden.")
 			else:
 				policy_problems.append(f"{str(user)} is not authorized to access this area at this time{details}. Creating, moving, and resizing reservations is forbidden.")
 
+	check_tool_reservation_requiring_area(policy_problems, user_creating_reservation, cancelled_reservation, new_reservation)
+
 	# The reservation start time may not exceed the item's reservation horizon.
 	# Staff may break this rule.
 	# An explicit policy override allows this rule to be broken.
@@ -243,12 +259,35 @@ def check_policy_to_save_reservation(cancelled_reservation: Optional[Reservation
 	# Check item policy rules
 	item_policy_problems = []
 	if should_enforce_policy(new_reservation):
-		item_policy_problems = check_policy_rules_for_item(cancelled_reservation, new_reservation, user_creating_reservation)
+		item_policy_problems = check_policy_rules_for_item(user_creating_reservation, new_reservation, cancelled_reservation)
 
 	# Return the list of all policies that are not met.
 	return policy_problems + item_policy_problems, overridable
 
 
+def check_tool_reservation_requiring_area(policy_problems: List[str], user_creating_reservation: User, cancelled_reservation: Optional[Reservation], new_reservation: Optional[Reservation]):
+	# When modifying an area reservation, check that all tools reservations starting during the cancelled one are still within the new one
+	if cancelled_reservation and cancelled_reservation.reservation_item_type == ReservationItemType.AREA:
+		tools_requiring_cancelled_reservation = Reservation.objects.filter(cancelled=False, missed=False, shortened=False, tool__isnull=False, tool___requires_area_access=cancelled_reservation.area, user=cancelled_reservation.user)
+		tools_requiring_cancelled_reservation = tools_requiring_cancelled_reservation.filter(start__gte=cancelled_reservation.start, start__lt=cancelled_reservation.end)
+		if tools_requiring_cancelled_reservation.exists():
+			if new_reservation:
+				tools_requiring_new_reservation = Reservation.objects.filter(cancelled=False, missed=False, shortened=False, tool__isnull=False, tool___requires_area_access=new_reservation.area, user=new_reservation.user)
+				tools_requiring_new_reservation = tools_requiring_new_reservation.filter(start__gte=new_reservation.start, start__lt=new_reservation.end)
+			else:
+				tools_requiring_new_reservation = Reservation.objects.none()
+			difference: List[Reservation] = [item for item in tools_requiring_cancelled_reservation if item not in tools_requiring_new_reservation]
+			# As long as the new reservation includes the same tool reservations, we are good
+			if difference:
+				user = cancelled_reservation.user
+				area = new_reservation.area if new_reservation else cancelled_reservation.area
+				difference.sort(key=lambda x: x.start)
+				if user == user_creating_reservation:
+					policy_problems.append(f"You have a reservation for the {difference[0].tool} at {format_datetime(difference[0].start)} that requires a {area} reservation. Cancel or reschedule the tool reservation first and try again.")
+				else:
+					policy_problems.append(f"{str(user)} has a reservation for the {difference[0].tool} at {format_datetime(difference[0].start)} that requires a {area} reservation. Cancel or reschedule the tool reservation first and try again.")
+
+
 def check_coincident_item_reservation_policy(cancelled_reservation: Optional[Reservation], new_reservation: Reservation, user_creating_reservation: User, policy_problems: List):
 	user = new_reservation.user
 
@@ -273,13 +312,20 @@ def check_coincident_item_reservation_policy(cancelled_reservation: Optional[Res
 				policy_problems.append(f"{str(user)} already has a reservation that coincides with this one. Please choose a different time.")
 		for area in new_reservation.area.get_ancestors(ascending=True, include_self=True):
 			# Check reservations for all other children of the parent areas
-			apply_to_user = not user.is_staff or user.is_staff and area.count_staff_in_occupancy
-			children_events = coincident_events.filter(area_id__in=[area.id for area in area.get_descendants(include_self=True)])
-			# Check only distinct users since the same user could make reservations in different rooms
-			distinct_users = set(children_events.values_list('user', flat=True).distinct())
-			distinct_users.add(user.id)
-			if apply_to_user and area.maximum_capacity and len(distinct_users) > area.maximum_capacity:
-				policy_problems.append(f"The {area} is already at its maximum capacity at this time. Please choose a different time.")
+			if not area.count_staff_in_occupancy:
+				coincident_events = coincident_events.filter(user__is_staff=False)
+			if not area.count_service_personnel_in_occupancy:
+				coincident_events = coincident_events.filter(user__is_service_personnel=False)
+			apply_to_user = (not user.is_staff and not user.is_service_personnel) or (user.is_staff and area.count_staff_in_occupancy) or (user.is_service_personnel and area.count_service_personnel_in_occupancy)
+			if apply_to_user and area.maximum_capacity:
+				children_events = coincident_events.filter(area_id__in=[area.id for area in area.get_descendants(include_self=True)])
+				reservations = list(children_events)
+				reservations.append(new_reservation)
+				# Check only distinct users since the same user could make reservations in different rooms
+				maximum_users, time = maximum_users_in_overlapping_reservations(reservations)
+				if maximum_users > area.maximum_capacity:
+					time_display = "at this time" if time is None else "at "+timezone.localtime(time).strftime('%I:%M %p')
+					policy_problems.append(f"The {area} would be over its maximum capacity {time_display}. Please choose a different time.")
 
 	# The user may not create, move, or resize a reservation to coincide with a scheduled outage.
 	if new_reservation.reservation_item_type == ReservationItemType.TOOL:
@@ -287,6 +333,8 @@ def check_coincident_item_reservation_policy(cancelled_reservation: Optional[Res
 			Q(tool=new_reservation.tool) | Q(resource__fully_dependent_tools__in=[new_reservation.tool]))
 	elif new_reservation.reservation_item_type == ReservationItemType.AREA:
 		coincident_events = new_reservation.area.scheduled_outage_queryset()
+	else:
+		coincident_events = ScheduledOutage.objects.none()
 	# Exclude events for which the following is true:
 	# The event starts and ends before the time-window, and...
 	# The event starts and ends after the time-window.
@@ -317,7 +365,7 @@ def should_enforce_policy(reservation: Reservation):
 	return should_enforce
 
 
-def check_policy_rules_for_item(cancelled_reservation: Optional[Reservation], new_reservation: Reservation, user_creating_reservation: User):
+def check_policy_rules_for_item(user_creating_reservation: User, new_reservation: Reservation, cancelled_reservation: Optional[Reservation]):
 	item_policy_problems = []
 	# Calculate the duration of the reservation:
 	duration = new_reservation.end - new_reservation.start
@@ -406,27 +454,39 @@ def check_policy_rules_for_item(cancelled_reservation: Optional[Reservation], ne
 	return item_policy_problems
 
 
-def check_policy_to_cancel_reservation(reservation, user_cancelling_reservation):
+def check_policy_to_cancel_reservation(user_cancelling_reservation: User, reservation_to_cancel: Reservation, new_reservation: Optional[Reservation] = None):
 	"""
 	Checks the reservation deletion policy.
 	If all checks pass the function returns an HTTP "OK" response.
 	Otherwise, the function returns an HTTP "Bad Request" with an error message.
 	"""
 
+	move = new_reservation and new_reservation.start != reservation_to_cancel.start
+	resize = new_reservation and new_reservation.start == reservation_to_cancel.start
+	action = 'move' if move else 'resize' if resize else 'cancel'
+
 	# Users may only cancel reservations that they own.
 	# Staff may break this rule.
-	if (reservation.user != user_cancelling_reservation) and not user_cancelling_reservation.is_staff:
-		return HttpResponseBadRequest("You may not cancel reservations that you do not own.")
+	if (reservation_to_cancel.user != user_cancelling_reservation) and not user_cancelling_reservation.is_staff:
+		return HttpResponseBadRequest(f"You may not {action} reservations that you do not own.")
 
 	# Users may not cancel reservations that have already ended.
 	# Staff may break this rule.
-	if reservation.end < timezone.now() and not user_cancelling_reservation.is_staff:
-		return HttpResponseBadRequest("You may not cancel reservations that have already ended.")
+	if reservation_to_cancel.end < timezone.now() and not user_cancelling_reservation.is_staff:
+		return HttpResponseBadRequest(f"You may not {action} reservations that have already ended.")
 
-	if reservation.cancelled:
-		return HttpResponseBadRequest("This reservation has already been cancelled by " + str(reservation.cancelled_by) + " at " + format_datetime(reservation.cancellation_time) + ".")
+	# Users may not cancel ongoing area reservations when they are currently logged in that area (unless they are extending it)
+	# Staff may break this rule.
+	if reservation_to_cancel.area and reservation_to_cancel.area.requires_reservation and not resize and reservation_to_cancel.start < timezone.now() < reservation_to_cancel.end and AreaAccessRecord.objects.filter(end=None, staff_charge=None, customer=reservation_to_cancel.user, area=reservation_to_cancel.area) and not user_cancelling_reservation.is_staff:
+		if move:
+			return HttpResponseBadRequest("You may only resize an area reservation while logged in that area.")
+		else:
+			return HttpResponseBadRequest("You may not cancel an area reservation while logged in that area.")
 
-	if reservation.missed:
+	if reservation_to_cancel.cancelled:
+		return HttpResponseBadRequest("This reservation has already been cancelled by " + str(reservation_to_cancel.cancelled_by) + " on " + format_datetime(reservation_to_cancel.cancellation_time) + ".")
+
+	if reservation_to_cancel.missed:
 		return HttpResponseBadRequest("This reservation was missed and cannot be modified.")
 
 	return HttpResponse()
@@ -476,9 +536,10 @@ def check_policy_to_enter_this_area(area:Area, user:User):
 	else:
 		# Check if the user normally has access to this area door at the current time (or access to any parent)
 		if not any([access_level.accessible() for access_level in user.accessible_access_levels_for_area(area)]):
-			raise NoAccessiblePhysicalAccessUserError(user=user, area=area)
+			first_access_exception = next(iter([access_level.ongoing_exception() for access_level in user.accessible_access_levels_for_area(area)]), None)
+			raise NoAccessiblePhysicalAccessUserError(user=user, area=area, access_exception=first_access_exception)
 
-	if not user.is_staff:
+	if not user.is_staff and not user.is_service_personnel:
 		for a in area.get_ancestors(ascending=True, include_self=True):
 			unavailable_resources = a.required_resources.filter(available=False)
 			if unavailable_resources:
@@ -496,3 +557,53 @@ def check_policy_to_enter_this_area(area:Area, user:User):
 		if area.requires_reservation and not area.get_current_reservation_for_user(user):
 			raise ReservationRequiredUserError(user=user, area=area)
 
+
+def maximum_users_in_overlapping_reservations(reservations: List[Reservation]) -> (int, datetime):
+	"""
+	Returns the maximum number of overlapping reservations and the earlier time the maximum is reached
+	This will only count reservations made by different users. i.e. if a user has 3 reservations at the same
+	time for different tools/areas, it will only count as one.
+	"""
+	# First we need to merge reservations by user, since one user could have more than one at the same time. (and we should only count it as one)
+	intervals_by_user = defaultdict(list)
+	for r in reservations:
+		intervals_by_user[r.user.id].append((r.start, r.end))
+
+	merged_intervals = []
+	for user, intervals in intervals_by_user.items():
+		merged_intervals.extend(recursive_merge(sorted(intervals).copy()))
+
+	# Now let's count the maximum overlapping reservations
+	times = []
+	for interval in merged_intervals:
+		start_time, end_time = interval[0], interval[1]
+		times.append((start_time, 'start'))
+		times.append((end_time, 'end'))
+	times = sorted(times)
+
+	count = 0
+	max_count = 0
+	max_time: Optional[datetime] = None
+	for time in times:
+		if time[1] == 'start':
+			count += 1  # increment on arrival/start
+		else:
+			count -= 1  # decrement on departure/end
+		# maintain maximum
+		prev_count = max_count
+		max_count = max(count, max_count)
+		# maintain earlier time max is reached
+		if max_count > prev_count:
+			max_time = time[0]
+	return max_count, max_time
+
+
+def recursive_merge(intervals: List[tuple], start_index=0) -> List[tuple]:
+	for i in range(start_index, len(intervals) - 1):
+		if intervals[i][1] > intervals[i + 1][0]:
+			new_start = intervals[i][0]
+			new_end = intervals[i + 1][1]
+			intervals[i] = (new_start, new_end)
+			del intervals[i + 1]
+			return recursive_merge(intervals.copy(), start_index=i)
+	return intervals
