@@ -1,33 +1,72 @@
-from datetime import timedelta
+import csv
+from datetime import timedelta, datetime
+from distutils.util import strtobool
 from http import HTTPStatus
 from itertools import chain
+from json import loads, JSONDecodeError
 from logging import getLogger
+from typing import Dict, Tuple, List
 
 from django.conf import settings
 from django.contrib.admin.views.decorators import staff_member_required
 from django.contrib.auth.decorators import login_required
 from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseNotFound, HttpResponseServerError
 from django.shortcuts import get_object_or_404, redirect, render
-from django.utils import timezone
+from django.utils import timezone, formats
 from django.views.decorators.http import require_GET, require_POST
 
 from NEMO import rates
 from NEMO.forms import CommentForm, nice_errors
-from NEMO.models import Comment, Configuration, ConfigurationHistory, Project, Reservation, StaffCharge, Task, TaskCategory, TaskStatus, Tool, UsageEvent, User
-from NEMO.utilities import extract_times, quiet_int
+from NEMO.models import Comment, Configuration, ConfigurationHistory, Project, Reservation, StaffCharge, Task, TaskCategory, TaskStatus, Tool, UsageEvent, User, ToolUsageCounter
+from NEMO.utilities import extract_times, quiet_int, send_mail
 from NEMO.views.calendar import shorten_reservation
 from NEMO.views.policy import check_policy_to_disable_tool, check_policy_to_enable_tool
 from NEMO.widgets.configuration_editor import ConfigurationEditor
-from NEMO.widgets.dynamic_form import DynamicForm
+from NEMO.widgets.dynamic_form import DynamicForm, PostUsageGroupQuestion, PostUsageQuestion
 from NEMO.widgets.item_tree import ItemTree
 
 tool_control_logger = getLogger(__name__)
 
+
+class PostUsageDataTable(object):
+	""" Utility table to make adding headers and rows easier """
+
+	def __init__(self):
+		self.headers: List[Tuple[str, str]] = []
+		self.rows: List[Dict] = []
+
+	def add_header(self, header: Tuple[str, str]):
+		if not any(k[0] == header[0] for k in self.headers):
+			self.headers.append((header[0], header[1].capitalize()))
+
+	def add_row(self, row:Dict):
+		self.rows.append(row)
+
+	def flat_headers(self) -> List[str]:
+		return [display for key, display in self.headers]
+
+	def flat_rows(self) -> List[List]:
+		flat_result = []
+		for row in self.rows:
+			flat_result.append([row.get(key, '') for key, display_value in self.headers])
+		return flat_result
+
+	def to_csv(self) -> HttpResponse:
+		response = HttpResponse(content_type='text/csv')
+		writer = csv.writer(response)
+		writer.writerow([display_value.capitalize() for key, display_value in self.headers])
+		for row in self.rows:
+			writer.writerow([row.get(key, '') for key, display_value in self.headers])
+		return response
+
+
+
 @login_required
 @require_GET
 def tool_control(request, tool_id=None):
 	""" Presents the tool control view to the user, allowing them to begin/end using a tool or see who else is using it. """
-	if request.user.active_project_count() == 0:
+	user: User = request.user
+	if user.active_project_count() == 0:
 		return render(request, 'no_project.html')
 	# The tool-choice sidebar is not available for mobile devices, so redirect the user to choose a tool to view.
 	if request.device == 'mobile' and tool_id is None:
@@ -39,7 +78,7 @@ def tool_control(request, tool_id=None):
 	}
 	# The tool-choice sidebar only needs to be rendered for desktop devices, not mobile devices.
 	if request.device == 'desktop':
-		dictionary['rendered_item_tree_html'] = ItemTree().render(None, {'tools': tools, 'user':request.user})
+		dictionary['rendered_item_tree_html'] = ItemTree().render(None, {'tools': tools, 'user': user})
 	return render(request, 'tool_control/tool_control.html', dictionary)
 
 
@@ -56,7 +95,7 @@ def tool_status(request, tool_id):
 		'rendered_configuration_html': tool.configuration_widget(request.user),
 		'mobile': request.device == 'mobile',
 		'task_statuses': TaskStatus.objects.all(),
-		'post_usage_questions': DynamicForm(tool.post_usage_questions).render(),
+		'post_usage_questions': DynamicForm(tool.post_usage_questions, tool.id).render(),
 		'configs': get_tool_full_config_history(tool),
 	}
 
@@ -108,6 +147,75 @@ def get_tool_full_config_history(tool: Tool):
 		)
 	return configs
 
+
+@login_required
+@require_POST
+def usage_data_history(request, tool_id):
+	""" This method return a dictionary of headers and rows containing run_data information for Usage Events """
+	csv_export = bool(request.POST.get('csv', False))
+	start, end = extract_times(request.POST, start_required=False, end_required=False)
+	if not start and not end:
+		# Default to last 30 days of usage data
+		end = timezone.now().astimezone(timezone.get_current_timezone())
+		start = end + timedelta(days=-30)
+	usage_events = UsageEvent.objects.filter(tool_id=tool_id, end__isnull=False).order_by('-end')
+	if start:
+		usage_events = usage_events.filter(end__gte=start.replace(hour=0, minute=0, second=0))
+	if end:
+		usage_events = usage_events.filter(end__lte=end.replace(hour=0, minute=0, second=0) + timedelta(days=1, minutes=-1))
+	table_result = PostUsageDataTable()
+	table_result.add_header(('user', 'User'))
+	table_result.add_header(('date', 'Date'))
+	for usage_event in usage_events:
+		if usage_event.run_data:
+			usage_data = {}
+			try:
+				user_data = f"{usage_event.user.first_name} {usage_event.user.last_name}"
+				date_data = usage_event.end.astimezone(timezone.get_current_timezone()).strftime('%m/%d/%Y @ %I:%M %p')
+				run_data: Dict = loads(usage_event.run_data)
+				for question_key, question in run_data.items():
+					if 'user_input' in question:
+						if question['type'] == 'group':
+							for sub_question in question['questions']:
+								table_result.add_header((sub_question['name'], sub_question['title']))
+							for index, user_inputs in question['user_input'].items():
+								if index == '0':
+									# Special case here the "initial" group of user inputs will go along with the rest of the non-group user inputs
+									for name, user_input in user_inputs.items():
+										usage_data[name] = user_input
+								else:
+									# For the other groups of user inputs, we have to add a whole new row
+									group_usage_data = {}
+									for name, user_input in user_inputs.items():
+										group_usage_data[name] = user_input
+									if group_usage_data:
+										group_usage_data['user'] = user_data
+										group_usage_data['date'] = date_data
+										table_result.add_row(group_usage_data)
+						else:
+							table_result.add_header((question_key, question['title']))
+							usage_data[question_key]=question['user_input']
+				if usage_data:
+					usage_data['user'] = user_data
+					usage_data['date'] = date_data
+					table_result.add_row(usage_data)
+			except JSONDecodeError:
+				tool_control_logger.debug("error decoding run_data: " + usage_event.run_data)
+	if csv_export:
+		response = table_result.to_csv()
+		filename = f"usage_data_{start.strftime('%m_%d_%Y')}_to_{end.strftime('%m_%d_%Y')}.csv"
+		response['Content-Disposition'] = f'attachment; filename="{filename}"'
+		return response
+	else:
+		dictionary = {
+			'tool_id': tool_id,
+			'data_history_start': start,
+			'data_history_end': end,
+			'usage_data_table': table_result,
+		}
+		return render(request, 'tool_control/usage_data.html', dictionary)
+
+
 @login_required
 @require_POST
 def tool_configuration(request):
@@ -196,6 +304,14 @@ def enable_tool(request, tool_id, user_id, project_id, staff_charge):
 	if tool.interlock and not tool.interlock.unlock():
 		raise Exception("The interlock command for this tool failed. The error message returned: " + str(tool.interlock.most_recent_reply))
 
+	# Start staff charge before tool usage
+	if staff_charge:
+		new_staff_charge = StaffCharge()
+		new_staff_charge.staff_member = request.user
+		new_staff_charge.customer = user
+		new_staff_charge.project = project
+		new_staff_charge.save()
+
 	# Create a new usage event to track how long the user uses the tool.
 	new_usage_event = UsageEvent()
 	new_usage_event.operator = operator
@@ -204,13 +320,6 @@ def enable_tool(request, tool_id, user_id, project_id, staff_charge):
 	new_usage_event.tool = tool
 	new_usage_event.save()
 
-	if staff_charge:
-		new_staff_charge = StaffCharge()
-		new_staff_charge.staff_member = request.user
-		new_staff_charge.customer = user
-		new_staff_charge.project = project
-		new_staff_charge.save()
-
 	return response
 
 
@@ -243,13 +352,15 @@ def disable_tool(request, tool_id):
 	current_usage_event.end = timezone.now() + downtime
 
 	# Collect post-usage questions
-	dynamic_form = DynamicForm(tool.post_usage_questions)
+	dynamic_form = DynamicForm(tool.post_usage_questions, tool.id)
 	current_usage_event.run_data = dynamic_form.extract(request)
 	dynamic_form.charge_for_consumables(current_usage_event.user, current_usage_event.operator, current_usage_event.project, current_usage_event.run_data)
+	dynamic_form.update_counters(current_usage_event.run_data)
 
 	current_usage_event.save()
-	if request.user.charging_staff_time():
-		existing_staff_charge = request.user.get_staff_charge()
+	user: User = request.user
+	if user.charging_staff_time():
+		existing_staff_charge = user.get_staff_charge()
 		if existing_staff_charge.customer == current_usage_event.user and existing_staff_charge.project == current_usage_event.project:
 			response = render(request, 'staff_charges/reminder.html', {'tool': tool})
 
@@ -300,3 +411,42 @@ def ten_most_recent_past_comments_and_tasks(request, tool_id):
 		'past': past,
 	}
 	return render(request, 'tool_control/past_tasks_and_comments.html', dictionary)
+
+
+@login_required
+@require_GET
+def tool_usage_group_question(request, tool_id, group_name):
+	tool = get_object_or_404(Tool, id=tool_id)
+	question_index = request.GET['index']
+	is_mobile = bool(strtobool((request.GET['is_mobile'])))
+	if tool.post_usage_questions:
+		for question in PostUsageQuestion.load_questions(loads(tool.post_usage_questions), tool.id, is_mobile, question_index):
+			if isinstance(question, PostUsageGroupQuestion) and question.group_name == group_name:
+				return HttpResponse(question.render_group_question())
+	return HttpResponse()
+
+
+@staff_member_required
+@require_GET
+def reset_tool_counter(request, counter_id):
+	counter = get_object_or_404(ToolUsageCounter, id=counter_id)
+	counter.last_reset_value = counter.value
+	counter.value = 0
+	counter.last_reset = datetime.now()
+	counter.last_reset_by = request.user
+	counter.save()
+
+	# Save a comment about the counter being reset.
+	comment = Comment()
+	comment.tool = counter.tool
+	comment.content = f"The {counter.name} counter was reset to 0. Its last value was {counter.last_reset_value}."
+	comment.author = request.user
+	comment.expiration_date = datetime.now() + timedelta(weeks=1)
+	comment.save()
+
+	# Send an email to Lab Managers about the counter being reset.
+	if hasattr(settings, 'LAB_MANAGERS'):
+		message = f"The {counter.name} counter for the {counter.tool.name} was reset to 0 on {formats.localize(counter.last_reset)} by {counter.last_reset_by}.<br/>" \
+				  f"Its last value was {counter.last_reset_value}."
+		send_mail(f'{counter.tool.name} counter reset', message, settings.SERVER_EMAIL, settings.LAB_MANAGERS)
+	return redirect('tool_control')
\ No newline at end of file
